package vc.thinker.absctacts.mqtt.protocol;


import org.eclipse.paho.client.mqttv3.MqttClient;
import vc.thinker.absctacts.mqtt.connection.MqttConnectionProperties;
import vc.thinker.absctacts.mqtt.publish.MqttCommandPush;

/**
 * mqtt 协议
 *
 * @author HeTongHao
 * @since 2019-10-29 16:00
 */
public interface IMqttProtocol {
    /**
     * 订阅Topic列表
     *
     * @return Topic列表
     */
    String[] subscribe();

    /**
     * 处理业务消息
     *
     * @param topic   监听到消息的Topic
     * @param message 监听到的消息
     */
    void handleMessage(String topic, byte[] message);

    /**
     * 得到消息推送
     *
     * @return 可以实现Mqtt消息推送
     */
    MqttCommandPush getMqttCommandPush();

    /**
     * 设置这个协议对应的连接
     *
     * @param mqttClient Mqtt连接
     */
    void setMqttClient(MqttClient mqttClient);

    /**
     * 获取Mqtt连接配置
     *
     * @return MqttConnectionProperties
     */
    MqttConnectionProperties getProperties();
}
